#include "AssetManager.h"
#include "Log.h"

#include <game-activity/native_app_glue/android_native_app_glue.h>

#define STBI_NO_STDIO
#define STB_IMAGE_IMPLEMENTATION
#include "stb_image.h"

AssetManager* AssetManager::ptr = nullptr;

AssetManager::AssetManager() : m_asset_mgr(nullptr) {};
AssetManager::~AssetManager() {}

void AssetManager::create(AAssetManager *android_asset_manager)
{
    AssetManager::ptr = new AssetManager;
    AssetManager::ptr->m_asset_mgr = android_asset_manager;
}

bool AssetManager::loadFile(const char* path, FileBuffer* p_file_buffer)
{
    AAsset* asset = AAssetManager_open(m_asset_mgr, path, AASSET_MODE_BUFFER);
    if (!asset) {
        ALOGE("Failed to open asset %s", path);
        return false;
    }
    const void* buffer = AAsset_getBuffer(asset);
    if (!buffer) {
        ALOGE("Failed to retrieve contents of asset %s", path);
        return false;
    }
    auto length = static_cast<size_t>(AAsset_getLength64(asset));
    p_file_buffer->data = buffer,
    p_file_buffer->size = length;
    p_file_buffer->internal = asset;
    return true;
}

bool AssetManager::loadTexture(const char* path, Texture* p_texture)
{
    FileBuffer texture_data = {};
    if (!loadFile(path, &texture_data))
        return false;

    int width, height, channels;
    const auto* raw_data = reinterpret_cast<const stbi_uc*>(texture_data.data);
    stbi_uc* data = stbi_load_from_memory(raw_data,
                          static_cast<int>(texture_data.size),
                          &width, &height, &channels,
                          4);
    if (!data) {
        ALOGE("Failed to parse texture file %s", path);
        releaseFileBuffer(texture_data);
        return false;
    }
    *p_texture = Texture(width, height, data);

    return true;
}

void AssetManager::releaseFileBuffer(FileBuffer &fb)
{
    AAsset_close(reinterpret_cast<AAsset*>(fb.internal));
    fb.internal = nullptr;
    fb.data = nullptr;
    fb.size = 0;
}